package	jp.co.sds_corp.nanchart.graph;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

/**
グラフ全体の抽象表現です。
各グラフは、このクラスに系列(GraphSeriesのサブクラス)を追加して実装します。
@author	小見山　聡
*/
public class Graph2D
{
	/** タイトルの位置：上 */
	public static int	TOP		= 0;
	/** タイトルの位置：下 */
	public static int	BOTTOM	= 1;

	/** 系列の保存用 */
	Vector<GraphSeries>	vecSrs = new Vector<GraphSeries>();
	/** X軸目盛り */
	GraphScale	xScale = null;
	/** Y軸目盛り */
	GraphScale	yScale = null;
	/** X軸目盛りの幅。X軸を明示的に指定しなかった時のみ使用します */
	int		xScaleHeight = 30;
	/** Y軸目盛りの幅。Y軸を明示的に指定しなかった時のみ使用します */
	int		yScaleWidth = 30;
	/** グラフの文字色 */
	Color	FgCol = Color.black;
	/** グラフの背景色 */
	Color	BgCol = Color.white;
	/** グラフのフォント */
	Font	font = new Font( "dialog", Font.PLAIN, 15 );
	/** グラフのタイトル */
	String	title = null;
	/** タイトルのフォント */
	Font	titlefont = new Font( "dialog", Font.PLAIN, 20 );
	/** タイトルの位置 */
	int		nTitlePos = TOP;
	/** ラベルの保存用 */
	Vector<Label>	vecLabel = new Vector<Label>();
	/** 描画アルゴリズムの推奨設定の値 */
	Hashtable<RenderingHints.Key,Object>	hashRender = new Hashtable<RenderingHints.Key,Object>();
	/** 出力先のイメージ */
	BufferedImage	img = null;
	Image		bgimg = null;

	/**
	* グラフを構築します。
	*/
	public Graph2D() {
	}

	/**
	* X軸・Y軸の範囲を指定してグラフを構築します。
	* この範囲は各系列の範囲とは無関係で、目盛りにのみ反映されます。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public Graph2D( double xMin, double xMax, double yMin, double yMax ) {
		xScale = new XScale( xMin, xMax );
		yScale = new YScale( yMin, yMax );
	}

	/**
	* X軸・Y軸の範囲を指定して、グラフを構築します。
	* 指定するオブジェクトのタイプについては、{@link Value}クラスの説明を
	* 参照してください。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public Graph2D( Object xMin, Object xMax, Object yMin, Object yMax ) {
		xScale = new XScale( xMin, xMax );
		yScale = new YScale( yMin, yMax );
	}

	/**
	* グラフに系列を追加します。
	* @param	dat		追加する系列
	* @return	追加された系列の順番。(0〜)
	*/
	public int addSeries( GraphSeries dat ) {
		dat.setFont( font);
		int		nRet = vecSrs.size();
		dat.setParent( this );
		vecSrs.add( dat );
		return nRet;
	}

	/**
	* 目盛りの範囲を指定してX軸の目盛りを構築します。
	* @param	min		X軸の最小値
	* @param	max		X軸の最大値
	*/
	public void setXScale( double min, double max ) {
		xScale = new XScale( min, max );
	}
	/**
	* X軸の目盛りを指定します。
	* @param	sc		目盛り。無い場合はnull。
	*/
	public void setXScale( GraphScale sc ) {
		xScale = sc;
	}
	/**
	* 目盛りの範囲を指定してY軸の目盛りを構築します。
	* @param	min		Y軸の最小値
	* @param	max		Y軸の最大値
	*/
	public void setYScale( double min, double max ) {
		yScale = new YScale( min, max );
	}
	/**
	* Y軸の目盛りを指定します。
	* @param	sc		目盛り。無い場合はnull。
	*/
	public void setYScale( GraphScale sc ) {
		yScale = sc;
	}
	/**
	* X軸の目盛りを取得します。
	* @return	X軸目盛りオブジェクト
	*/
	public GraphScale getXScale() {
		return xScale;
	}
	/**
	* Y軸の目盛りを取得します。
	* @return	Y軸目盛りオブジェクト
	*/
	public GraphScale getYScale() {
		return yScale;
	}

	/**
	* X軸目盛りの高さを設定します。
	* この設定は、グラフ出力時にグラフの大きさを指定しなかったときのみ有効です。
	* @param	nHeight	X軸目盛りの高さ。
	* @see		#getImage(int,int)
	*/
	public void setXScaleHeight( int nHeight ) {
		xScaleHeight = nHeight;
	}
	/**
	* Y軸目盛りの幅を設定します。
	* この設定は、グラフ出力時にグラフの大きさを指定しなかったときのみ有効です。
	* @param	nWidth	Y軸目盛りの幅。
	* @see		#getImage(int,int)
	*/
	public void setYScaleWidth( int nWidth ) {
		yScaleWidth = nWidth;
	}

	/**
	* グラフで使用するフォントを設定します。
	* @param	font	設定するフォント。
	*/
	public void setFont( Font font ) {
		this.font = font;
		xScale.setFont( font );
		yScale.setFont( font );
	}
	/**
	* グラフで現在使用しているフォントを返します。
	* @return	現在のフォント。
	*/
	public Font getFont() {
		return font;
	}

	/**
	* グラフの文字色を設定します。
	* @param	col		設定する背景色。
	*/
	public void setColor( Color col ) {
		this.FgCol = col;
	}

	/**
	* グラフの背景色を設定します。
	* @param	col		設定する背景色。
	*/
	public void setBackground( Color col ) {
		this.BgCol = col;
	}

	/**
	* 描画アルゴリズムの推奨設定の値を 1 つ設定します。
	* @param	hintKey		設定されるヒントのキー
	* @param	hintValue	指定されたヒントカテゴリの推奨設定を示す値
	*/
	public void setRenderingHint( RenderingHints.Key hintKey, Object hintValue ) {
		hashRender.put( hintKey, hintValue );
	}

	/**
	* 書き込みを行うイメージを設定します。
	* 一度出力を行ったグラフ、もしくは全く別のイメージの上にグラフを
	* 書き込む時に使用します。
	* @param	img		イメージ
	*/
	public void setImage( BufferedImage img ) {
		this.img = img;
	}

	/**
	* グラフのタイトルを設定します。
	* @param	title	設定するタイトル。
	*/
	public void setTitle( String title ) {
		this.title = title;
	}
	/**
	* グラフのタイトルを返します。
	* @return	現在のタイトル。
	*/
	public String getTitle() {
		return title;
	}

	/**
	* タイトルに使用するフォントを設定します。
	* @param	font	設定するフォント。
	*/
	public void setTitleFont( Font font ) {
		this.titlefont = font;
	}
	/**
	* タイトルに使用するフォントを返します。
	* @return	現在のフォント。
	*/
	public Font getTitleFont() {
		return titlefont;
	}

	/**
	* タイトルの位置を設定します。
	* @param	nPos	タイトルの位置
	* @see		#TOP
	* @see		#BOTTOM
	*/
	public void setTitlePosition( int nPos ) {
		this.nTitlePos = nPos;
	}

	/**
	* 背景の画像を設定します。
	* @param	img		背景の画像
	*/
	public void setBgImage( Image img ) {
		bgimg = img;
	}

	/**
	* グラフ上に描画するラベルの情報を保持します。
	*/
	protected class Label
	{
		/** X座標の値 */
		Object	x;
		/** Y座標の値 */
		Object	y;
		/** 文字列 */
		String	msg;
		/** 文字色 */
		Color	fg = null;
		/** 背景色 */
		Color	bg = null;
		/** フォント */
		Font	font = null;

		/**
		* ラベルオブジェクトを、指定された値で構築します。
		* @param	x		X座標の値
		* @param	y		Y座標の値
		* @param	msg		文字列
		* @param	fg		文字色
		* @param	bg		背景色
		* @param	font	フォント
		*/
		public Label( Object x, Object y, String msg, Color fg, Color bg, Font font ) {
			this.x = x;
			this.y = y;
			this.msg = msg;
			this.fg = fg;
			this.bg = bg;
			this.font = font;
		}

		/**
		* X座標の値を返します
		* @return	X座標の値
		*/
		public Object getX() {
			return this.x;
		}
		/**
		* Y座標の値を返します
		* @return	Y座標の値
		*/
		public Object getY() {
			return this.y;
		}
		/**
		* 設定されている文字列を返します。
		* @return	文字列
		*/
		public String getMsg() {
			return this.msg;
		}
		/**
		* 設定されている文字色を返します。
		* @return	文字色
		*/
		public Color getColor() {
			return this.fg;
		}
		/**
		* 設定されている背景色を返します。
		* @return	背景色
		*/
		public Color getBackground() {
			return this.bg;
		}
		/**
		* 設定されているフォントを返します。
		* @return	フォント
		*/
		public Font getFont() {
			return this.font;
		}
	}

	/**
	* グラフにラベルを追加します。
	* @param	x		ラベルのX座標の値
	* @param	y		ラベルのY座標の値
	* @param	msg		ラベルの文字列
	*/
	public void addLabel( double x, double y, String msg ) {
		addLabel( x, y, msg, null, null, null );
	}
	/**
	* グラフにラベルを追加します。
	* @param	x		ラベルのX座標の値
	* @param	y		ラベルのY座標の値
	* @param	msg		ラベルの文字列
	*/
	public void addLabel( Object x, Object y, String msg ) {
		addLabel( x, y, msg, null, null, null );
	}
	/**
	* グラフにラベルを追加します。
	* @param	x		ラベルのX座標の値
	* @param	y		ラベルのY座標の値
	* @param	msg		ラベルの文字列
	* @param	fg		文字色
	* @param	bg		背景色
	* @param	font	フォント
	*/
	public void addLabel( Object x, Object y, String msg, Color fg, Color bg, Font font ) {
		if ( fg == null ) {
			fg = FgCol;
		}
		vecLabel.add( new Label( x, y, msg, fg, bg, font ));
	}
	/**
	* グラフにラベルを追加します。
	* @param	x		ラベルのX座標の値
	* @param	y		ラベルのY座標の値
	* @param	msg		ラベルの文字列
	* @param	fg		文字色
	* @param	bg		背景色
	* @param	font	フォント
	*/
	public void addLabel( double x, double y, String msg, Color fg, Color bg, Font font ) {
		addLabel( Double.valueOf( x ), Double.valueOf( y ), msg, fg, bg, font );
	}

	/**
	* グラフ上に設定されたラベルを描画します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	protected void drawLabel( Graphics2D gr, Rectangle rect ) {
		Label	lbl;
		Rectangle2D recmsg;
		for ( int nLoop = 0; nLoop < vecLabel.size(); nLoop++ ) {
			lbl = (Label )vecLabel.get( nLoop );
			recmsg = font.getStringBounds( lbl.getMsg(), 0, lbl.getMsg().length(),
										gr.getFontRenderContext() );
			recmsg.setRect(
					rect.getX() + xScale.getPos( rect.getWidth(), lbl.getX() ),
					rect.getY()
					+ ( rect.getHeight()
						- yScale.getPos( rect.getHeight(), lbl.getY() ) ),
					recmsg.getWidth(),
					recmsg.getHeight() );
			if ( lbl.getFont() == null ) {
				gr.setFont( font );
			} else {
				gr.setFont( lbl.getFont() );
			}
			if ( lbl.getBackground() != null ) {
				gr.setColor( lbl.getBackground() );
				gr.fill( recmsg );
			}
			gr.setColor( lbl.getColor() );
			gr.drawString( lbl.getMsg(), (int)recmsg.getX(),
				(int)( recmsg.getY() + recmsg.getHeight() ));
		}
	}

	/**
	* グラフのタイトルを描画します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	protected void drawTitle( Graphics2D gr, Rectangle rect ) {
		if ( title == null ) {
			return;
		}
		gr.setColor( FgCol );
		gr.setFont( titlefont );
		Rectangle2D recmsg = titlefont.getStringBounds(
								title, 0, title.length(),
								gr.getFontRenderContext() );
		if ( nTitlePos == TOP ) {
			gr.drawString(
				title,
				(int )( rect.getX() + rect.getWidth()/2 - recmsg.getWidth()/2 ),
				(int )( rect.getY() ) - 5 );
		} else {
			gr.drawString(
				title,
				(int )( rect.getX() + rect.getWidth()/2 - recmsg.getWidth()/2 ),
				(int )( rect.getY() + rect.getHeight() + recmsg.getHeight()) - 5 );
		}
	}

	/**
	* グラフのイメージを取得します。
	* setImage()により別イメージが設定されている場合には幅と高さの指定は
	* 無視されます。
	* 新規で作成する場合、{@link #setXScaleHeight(int)},
	* {@link #setYScaleWidth(int)}で指定された目盛りの幅を差し引いてグラフ本体を
	* 出力します。
	* @param	nWidth	出力するイメージの幅。
	* @param	nHeight	出力するイメージの高さ。
	* @return	作成されたイメージ
	* @see		#setXScaleHeight(int)
	* @see		#setYScaleWidth(int)
	*/
	public BufferedImage getImage( int nWidth, int nHeight ) {
		return getImage( nWidth, nHeight,
				new Rectangle( yScaleWidth, 0, nWidth-yScaleWidth,
								nHeight-xScaleHeight ));
	}

	/**
	* グラフのイメージを取得します。
	* setImage()により別イメージが設定されている場合には幅と高さの指定は
	* 無視されます。
	* 作成されるグラフ本体の大きさは{@link Rectangle}で指定された大きさになり
	* 目盛りはグラフ本体の外に描画されます。
	* このメソッドで出力されるイメージは{@link BufferedImage#TYPE_BYTE_BINARY}
	* になります。
	* @param	nWidth	出力するイメージの幅。
	* @param	nHeight	出力するイメージの高さ。
	* @param	rect	出力するグラフ領域の境界の矩形。
	* @return	作成されたイメージ
	*/
	public BufferedImage getImage( int nWidth, int nHeight, Rectangle rect ) {
		return getImage( nWidth, nHeight, rect, BufferedImage.TYPE_BYTE_BINARY);
	}

	/**
	* グラフのイメージを取得します。
	* {@link #setImage(BufferedImage)}により別イメージが設定されている場合には
	* 幅と高さの指定は無視されます。
	* 作成されるグラフ本体の大きさは{@link Rectangle}で指定された大きさになり
	* 目盛りはグラフ本体の外に描画されます。
	* @param	nWidth	出力するイメージの幅。
	* @param	nHeight	出力するイメージの高さ。
	* @param	rect	出力するグラフ領域の境界の矩形。
	* @param	imgType	出力するイメージのタイプ。
	*					{@link BufferedImage}の定義済みイメージタイプを参照して
	*					ください。
	* @return	作成されたイメージ
	*/
	public BufferedImage getImage( int nWidth, int nHeight, Rectangle rect,
									int imgType ) {
		Graphics2D		gr;
		if ( img == null ) {
			img = new BufferedImage( nWidth, nHeight, imgType );
			gr = img.createGraphics();
			gr.setColor( BgCol );
			gr.fillRect( 0,0, nWidth, nHeight );
		} else {
			gr = img.createGraphics();
		}
		if ( bgimg != null ) {
			gr.drawImage( bgimg, new AffineTransform(), null );
		}
		drawGraph( gr, rect );
		return img;
	}

	/**
	* グラフを描画します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public void drawGraph( Graphics2D gr, Rectangle rect ) {
		Enumeration<RenderingHints.Key> en = hashRender.keys();
		while ( en.hasMoreElements() ) {
			RenderingHints.Key key = (RenderingHints.Key )en.nextElement();
			Object value = hashRender.get( key );
			gr.setRenderingHint( key, value );
		}
		gr.setColor( FgCol );

		if ( xScale != null ) {
			xScale.drawScaleLine( gr, rect );
			xScale.drawScale( gr, rect );
		}
		if ( yScale != null ) {
			yScale.drawScaleLine( gr, rect );
			yScale.drawScale( gr, rect );
		}
		for ( int nLoop = 0; nLoop < vecSrs.size(); nLoop++ ) {
			GraphSeries dat = (GraphSeries )vecSrs.get( nLoop );
			dat.drawScaleLine( gr, rect );
			dat.drawScale( gr, rect );
		}

		for ( int nLoop = 0; nLoop < vecSrs.size(); nLoop++ ) {
			GraphSeries dat = (GraphSeries )vecSrs.get( nLoop );
			gr.setClip( rect );
			dat.drawGraph( gr, rect );
			gr.setClip( null );
		}

		drawLabel( gr, rect );
		drawTitle( gr, rect );

	}
}

