package	jp.co.sds_corp.nanchart.graph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.util.TreeMap;

/**
GraphScaleは、グラフの目盛りの抽象基底クラスです。
このクラスのサブクラスでは{@link #drawScale(Graphics2D,Rectangle)}メソッドで
目盛りをイメージに出力します。
@see	Graph2D
@see	Value
*/
public abstract class GraphScale extends TreeMap
{
	private static final long serialVersionUID = 1L;

	/** 描画位置：左、もしくは下 */
	public static int	LEFT_BOTTOM = 0;
	/** 描画位置：右、もしくは上 */
	public static int	RIGHT_TOP = 1;
	/** 描画方向：外向き */
	public static int	DIR_OUTER = 0;
	/** 描画方向：内向き */
	public static int	DIR_INNER = 1;

	/** 目盛りの最小値 */
	Object		min = null;
	/** 目盛りの最大値 */
	Object		max = null;
	/** 目盛りを描画する最大値 */
	Object		drawmax = null;
	/** 目盛りの描画する最小値 */
	Object		drawmin = null;
	/** 描画位置 */
	int			nPos = LEFT_BOTTOM;
	/** 描画方向 */
	int			nDir = DIR_OUTER;
	/** 描画に使用するフォント */
	Font		font = new Font( "dialog", Font.PLAIN, 15 );
	/** 描画に使用する線種 */
	Stroke		str = new BasicStroke( 1 );
	/** 描画に使用する色 */
	Color		col = Color.black;
	/** 描画に使用する線種 */
	Stroke		linestr = new BasicStroke( 1 );
	/** 描画に使用する色 */
	Color		linecol = Color.black;
	/** 目盛り文字列の角度 */
	double		ang = 0;
	/** 目盛りの長さ */
	int			ScaleLen = 5;
	/** 目盛り線出力フラグ */
	boolean		bLine = false;
	/** グラフ出力領域 */
	Rectangle	rect;

	/**
	* デフォルトの設定でGraphScaleを構築します。
	*/
	public GraphScale() {
	}

	/**
	* 値の範囲を指定して、GraphScaleを構築します。
	* @param	min		最小値
	* @param	max		最大値
	*/
	public GraphScale( Object min, Object max ) {
		setRange( min,  max );
	}

	/**
	* 値の範囲を指定して、GraphScaleを構築します。
	* @param	min		最小値
	* @param	max		最大値
	*/
	public GraphScale( double min, double max ) {
		setRange( Double.valueOf( min ),  Double.valueOf( max ));
	}

	/**
	* 目盛りの値の範囲を設定します。
	* @param	min		最小値
	* @param	max		最大値
	*/
	public void setRange( Object min, Object max ) {
		this.min = min;
		this.max = max;
	}

	/**
	* 目盛りの最大値を返します。
	* @return	最大値
	*/
	public Object getMax() {
		return max;
	}

	/**
	* 目盛りの最小値を返します。
	* @return	最小値
	*/
	public Object getMin() {
		return min;
	}

	/**
	* 目盛りを描画する値の範囲を設定します。
	* @param	min		最小値
	* @param	max		最大値
	*/
	public void setDrawRange( Object min, Object max ) {
		this.drawmin = min;
		this.drawmax = max;
	}

	/**
	* 目盛りを描画する値の範囲を設定します。
	* @param	nPos	位置
	* @see		#LEFT_BOTTOM
	* @see		#RIGHT_TOP
	*/
	public void setPosition( int nPos ) {
		this.nPos = nPos;
	}

	/**
	* 目盛りを描画する値の方向を設定します。
	* @param	nDir	方向
	* @see		#DIR_OUTER
	* @see		#DIR_INNER
	*/
	public void setDirection( int nDir ) {
		this.nDir = nDir;
	}

	/**
	* 目盛りにデータを追加します。
	* @param	obj		目盛りの位置を表す値
	* @param	msg		目盛りに表示する文字列
	*/
	public void add( Object obj, String msg ) {
		put( obj, msg );
	}

	/**
	* 目盛りにデータを追加します。
	* @param	x		目盛りの位置を表す値
	* @param	msg		目盛りに表示する文字列
	*/
	public void add( double x, String msg ) {
		put( new Double( x ), msg );
	}

	/**
	* 目盛りの値を座標値に変換します。
	* @param	dAll	全体の値
	* @param	pos		目盛りを表示するオブジェクト
	* @return	目盛りの座標値
	*/
	public double getPos( double dAll, Object pos ) {
		return new Value( pos ).diff( min )
				/ new Value( max ).diff( min ) * dAll;
	}

	/**
	* 描画に使用するフォントを設定します。
	* @param	font	フォント
	*/
	public void setFont( Font font ) {
		this.font = font;
	}

	/**
	* 描画に使用するフォントを返します。
	* @return	フォント
	*/
	public Font getFont() {
		return this.font;
	}

	/**
	* 描画に使用する線種を設定します。
	* @param	str		線種
	*/
	public void setStroke( Stroke str ) {
		this.str = str;
	}

	/**
	* 描画に使用する線種を返します。
	* @return	線種
	*/
	public Stroke getStroke() {
		return this.str;
	}

	/**
	* 描画に使用する色を設定します。
	* @param	col		色
	*/
	public void setColor( Color col ) {
		this.col = col;
	}

	/**
	* 描画に使用する色を返します。
	* @return	色
	*/
	public Color getColor() {
		return this.col;
	}

	/**
	* 描画に使用する線種を設定します。
	* @param	str		線種
	*/
	public void setLineStroke( Stroke str ) {
		this.linestr = str;
	}

	/**
	* 描画に使用する線種を返します。
	* @return	線種
	*/
	public Stroke getLineStroke() {
		return this.linestr;
	}

	/**
	* 描画に使用する色を設定します。
	* @param	col		線の色
	*/
	public void setLineColor( Color col ) {
		this.linecol = col;
	}

	/**
	* 描画に使用する色を返します。
	* @return	線の色
	*/
	public Color getLineColor() {
		return this.linecol;
	}

	/**
	* 目盛り線を出力するかを設定します。
	* @param	bFlg	フラグ
	*/
	public void setScaleLine( boolean bFlg ) {
		this.bLine = bFlg;
	}

	/**
	* 文字列の回転する角度を設定します。
	* @param	ang		回転する角度。0度は3時の方向で、反時計回りに指定する。
	*/
	public void rotate( double ang ) {
		this.ang = ang;
	}

	/**
	* 目盛り線をイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public abstract void drawScaleLine( Graphics2D gr, Rectangle rect );

	/**
	* 目盛りをイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public abstract void drawScale( Graphics2D gr, Rectangle rect );

}

