package	jp.co.sds_corp.nanchart.graph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.geom.AffineTransform;
import java.awt.geom.Arc2D;
import java.awt.geom.Rectangle2D;
import java.util.Iterator;

/**
PieGraphは、円グラフを実装します。
データ格納クラスに格納するデータは、順番を表すkeyオブジェクトと
円グラフの１片を表現するPieDataでなくてはなりません。
@see	PieData
*/
public class PieGraph extends GraphSeries
{
	private static final long serialVersionUID = 1L;

	/** 方向：時計回り */
	public static int	DIR_CLOCKWISE = 0;
	/** 方向：反時計回り */
	public static int	DIR_ANTICLOCK = 1;
	/** 円グラフの方向 */
	int		nDir = DIR_CLOCKWISE;
	/** ラベル：外周 */
	public static int	LBL_OUTER = 0;
	/** ラベル：中心に向かって */
	public static int	LBL_TOCENTER = 1;
	int		nLblMode = LBL_OUTER;
	/** 色のデフォルト値 */
	Color[]	BgCol = {
		Color.blue, Color.cyan, Color.gray, Color.magenta, Color.orange,
		Color.pink, Color.red, Color.yellow
	};
	/** 開始角度 */
	int		nStartAng = 90;
	/** 全体の角度 */
	int		nMaxAng = 360;
	/** 分割円フラグ */
	boolean	bExp = false;
	/** 分割円の半径 */
	int		nExp = 20;

	/**
	* デフォルトの状態で円グラフを構築します。
	*/
	public PieGraph() {
		setStorage( new PieStorage());
	}

	/**
	* X軸とY軸の範囲を指定して、円グラフを構築します。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public PieGraph( double xMin, double xMax, double yMin, double yMax ) {
		setStorage( new PieStorage( xMin, xMax, yMin, yMax ) );
	}

	/**
	* X軸とY軸の範囲を指定して、円グラフを構築します。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public PieGraph( Object xMin, Object xMax, Object yMin, Object yMax ) {
		setStorage( new PieStorage( xMin, xMax, yMin, yMax ) );
	}

	/**
	* データ格納クラスを指定して、円グラフを構築します。
	* @param	map		データ格納クラス
	*/
	public PieGraph( GraphStorage map ) {
		super( map );
	}

	/**
	* 開始角度を設定します。
	* 角度は 3時の位置を 0度として変換され、正の値は反時計方向、負の値は
	* 時計方向の回転を示します。
	* @param	ang		開始角度
	*/
	public void setStartAng( int ang ) {
		this.nStartAng = ang;
	}

	/**
	* 開始角度を取得します。
	* @return	開始角度
	*/
	public int getStartAng() {
		return this.nStartAng;
	}

	/**
	* 円グラフの向きを設定します。
	* @param	dir		方向
	* @see	#DIR_CLOCKWISE
	* @see	#DIR_ANTICLOCK
	*/
	public void setDirection( int dir ) {
		this.nDir = dir;
	}

	/**
	* 円グラフの向きを取得します。
	* @return	方向
	*/
	public int getDirection() {
		return this.nDir;
	}

	/**
	* 円グラフの最大角度を設定します。
	* @param	ang		最大角度
	*/
	public void setMaxAng( int ang ) {
		this.nMaxAng = ang;
	}

	/**
	* 円グラフの最大角度を取得します。
	* @return	最大角度
	*/
	public int getMaxAng() {
		return this.nMaxAng;
	}

	/**
	* データ格納クラスからデータを取得します。
	* @param	nIdx	取得するデータのインデックス
	* @return	PieData
	*/
	public PieData getData( int nIdx ) {
		return (PieData )store.get( Integer.valueOf( nIdx ));
	}

	/**
	* データ格納クラスに格納されているデータに、メッセージを設定します。
	* @param	nIdx	取得するデータのインデックス
	* @param	msg		メッセージ
	*/
	public void setMsg( int nIdx, String msg ) {
		((PieData)store.get( Integer.valueOf( nIdx ) )).setMsg( msg );
	}

	/**
	* データ格納クラスに格納されているデータに、色を設定します。
	* @param	nIdx	取得するデータのインデックス
	* @param	col		色
	*/
	public void setPieColor( int nIdx, Color col ) {
		((PieData)store.get( Integer.valueOf( nIdx ) )).setColor( col );
	}

	/**
	* データ格納クラスに格納されているデータから、色を取得します。
	* 色が設定されていない場合には、デフォルトの色を設定します。
	* @param	nIdx	取得するデータのインデックス
	*/
	public Color getPieColor( int nIdx ) {
		Color col;
		PieData dat = (PieData)store.get( Integer.valueOf( nIdx ) );
		if ( dat == null ) {
			col = BgCol[nIdx % BgCol.length ];
		} else {
			col = dat.getColor();
			if ( col == null ) {
				col = BgCol[nIdx % BgCol.length ];
			}
		}
		return col;
	}

	/**
	* データ格納クラスに格納されているデータに、分割円フラグを設定します。
	* このフラグがtrueに設定されたデータの部分円は、
	* 分割円の半径だけ元の中心から離れたところに描画されます。
	* @param	nIdx	取得するデータのインデックス
	* @param	bExp	分割円フラグ
	*/
	public void setExploded( int nIdx, boolean bExp ) {
		((PieData)store.get( Integer.valueOf( nIdx ) )).setExploded( bExp );
	}

	/**
	* この円グラフ全体を分割円グラフに設定します。
	* 全ての部分円は、分割円の半径だけ元の中心から離れたところに描画されます。
	* @param	bExp	分割円フラグ
	*/
	public void setExploded( boolean bExp ) {
		this.bExp = bExp;
	}

	/**
	* 分割円の半径を設定します。
	* 全ての部分円は、分割円の半径だけ元の中心から離れたところに描画されます。
	* @param	nSize	分割円の半径
	*/
	public void setExpSize( int nSize ) {
		this.nExp = nSize;
	}

	/**
	* ラベルの表示方法を設定します。
	* @param	nMode	表示方法
	* @see		#LBL_OUTER
	* @see		#LBL_TOCENTER
	*/
	public void setLblMode( int nMode ) {
		this.nLblMode = nMode;
	}

	/**
	* 円グラフにデータを追加します。
	* @param	nIdx	追加するデータのインデックス
	* @param	y		データ
	*/
	public void add( int nIdx, double y ) {
		add( nIdx, y, null, null );
	}

	/**
	* 円グラフにデータを追加します。
	* @param	nIdx	追加するデータのインデックス
	* @param	y		データ
	* @param	msg		メッセージ
	*/
	public void add( int nIdx, double y, String msg ) {
		add( nIdx, y, msg, null );
	}

	/**
	* 円グラフにデータを追加します。
	* @param	nIdx	追加するデータのインデックス
	* @param	y		データ
	* @param	msg		メッセージ
	* @param	piecol	部分円の色
	*/
	public void add( int nIdx, double y, String msg, Color piecol ) {
		add( nIdx, y, msg, null, bExp );
	}

	/**
	* 円グラフにデータを追加します。
	* @param	nIdx	追加するデータのインデックス
	* @param	y		データ
	* @param	msg		メッセージ
	* @param	piecol	部分円の色
	* @param	bExp	分割円フラグ
	*/
	public void add( int nIdx, double y, String msg, Color piecol, boolean bExp ) {
		store.add( Integer.valueOf( nIdx ), new PieData( y, msg, piecol, bExp ) );
	}

	private double	dSum = 0.0;
	private int		nPos = 0;
	private double	sta = 0.0;

	/**
	* グラフをイメージに出力します。
	* @param	gr		出力先
	* @param	recgr	出力先の矩形
	*/
	public void drawGraph( Graphics2D gr, Rectangle recgr) {
		if ( store.getYMax() == null ) {
			store.setYRange();
		}

		gr.setStroke( str );

		Iterator	itr;
		Object		curX, preX;
		Object		curY, preY;
		sta = nStartAng;

		itr = store.keySet().iterator();
		while ( itr.hasNext() ) {
			curX = itr.next();
			curY = store.get( curX );

			drawPoint( gr, recgr, null, curX, null, curY, null, null, null );

			preX = curX;
			preY = curY;
		}
	}

	/**
	* グラフ描画を行ないます。
	* @param	gr		描画するデバイス
	* @param	rect	描画する矩形
	* @param	srs		描画するグラフ
	* @param	curX	現在のX座標
	* @param	curY1	現在のY座標1
	* @param	curY2	現在のY座標2
	* @param	preX	前回のX座標
	* @param	preY1	前回のY座標1
	* @param	preY2	前回のY座標2
	*/
	public void drawPoint( Graphics2D gr, Rectangle rect, Object srs,
				Object curX, Object curY1, Object curY2,
				Object preX, Object preY1, Object preY2 )
	{
		float	w = ((BasicStroke)str).getLineWidth();
		Rectangle2D.Double recpie = new Rectangle2D.Double(
								w/2+nExp+rect.getX(), w/2+nExp+rect.getY(),
								rect.getWidth()-w-nExp*2, rect.getHeight()-w-nExp*2 );
		Rectangle2D.Double recexp = new Rectangle2D.Double(
								rect.getWidth()/2 - nExp + rect.getX(),
								rect.getHeight()/2 - nExp + rect.getY(),
								nExp*2, nExp*2 );
		Rectangle2D	recnew;
		Rectangle2D rectmp;
		Arc2D.Double	arc = new Arc2D.Double();
		Arc2D.Double	arcexp = new Arc2D.Double();
		double	ang = 0;
		double	rot = 0;
		String	msg;
		PieData	curY = null;

		curY = (PieData )curY2;
		msg = (String )curY.getMsg();
		Rectangle	recstor = new Rectangle( 0, 0, 0, nMaxAng );
		ang = store.getYPos( recstor, null, curY );
		if ( nDir == DIR_CLOCKWISE ) {
			ang = 0 - ang;
		}

		if ( curY.getExploded() ) {
			arcexp.setArc( recexp, sta+ang/2, 1, Arc2D.OPEN );
			rectmp = arcexp.getBounds2D();
			recnew = new Rectangle2D.Double(
							w/2+nExp+rectmp.getX()-rect.getWidth()/2+10,
							w/2+nExp+rectmp.getY()-rect.getHeight()/2+10,
							rect.getWidth()-w-nExp*2-20, rect.getHeight()-w-nExp*2-20 );
		} else {
			recnew = recpie;
		}
		arc.setArc( recnew, sta, ang, Arc2D.PIE );

		gr.setColor( getPieColor( nPos ));
		gr.fill( arc );
		gr.setColor( LineCol );
		gr.draw( arc );

		if ( msg != null ) {
			AffineTransform	at = new AffineTransform();
			rot = 450 - sta - ang/2;
			if ( nLblMode == LBL_OUTER ) {
				rot = 450 - sta - ang/2;
			} else {
				rot = 450 - sta - ang/2 + 90;
			}
			at.rotate( Math.toRadians( rot ));
			Font f = font.deriveFont( at );
			gr.setFont( f );
			gr.setColor( Color.black );
			arc.setArc( recnew, sta + ang/2, 1, Arc2D.OPEN );
			Rectangle2D recarc = arc.getBounds2D();
			gr.drawString( msg, (int )recarc.getCenterX(), (int )recarc.getCenterY() );
		}
		sta += ang;
		nPos++;
		dSum += curY.getData();
	}
}

